@@ -26,6 +26,7 @@
 #endif
 
 #include <gio/gio.h>
+#include <gio/gdesktopappinfo.h>
 #include "drive-button.h"
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
@@ -52,7 +53,7 @@ static void     drive_button_set_mount    (DriveButton    *self,
 static void     drive_button_reset_popup  (DriveButton    *self);
 static void     drive_button_ensure_popup (DriveButton    *self);
 
-static void     drive_button_destroy      (GtkObject      *object);
+static void     drive_button_dispose      (GObject      *object);
 #if 0
 static void     drive_button_unrealize    (GtkWidget      *widget);
 #endif /* 0 */
@@ -66,7 +67,7 @@ static void drive_button_theme_change     (GtkIconTheme   *icon_theme,
 static void
 drive_button_class_init (DriveButtonClass *class)
 {
-    GTK_OBJECT_CLASS(class)->destroy = drive_button_destroy;
+    G_OBJECT_CLASS(class)->dispose = drive_button_dispose;
     GTK_WIDGET_CLASS(class)->button_press_event = drive_button_button_press;
     GTK_WIDGET_CLASS(class)->key_press_event = drive_button_key_press;
 
@@ -133,7 +134,7 @@ drive_button_new_from_mount (GMount *mount)
 }
 
 static void
-drive_button_destroy (GtkObject *object)
+drive_button_dispose (GObject *object)
 {
     DriveButton *self = DRIVE_BUTTON (object);
 
@@ -145,8 +146,7 @@ drive_button_destroy (GtkObject *object)
 
     drive_button_reset_popup (self);
 
-    if (GTK_OBJECT_CLASS (drive_button_parent_class)->destroy)
-	(* GTK_OBJECT_CLASS (drive_button_parent_class)->destroy) (object);
+    G_OBJECT_CLASS (drive_button_parent_class)->dispose (object);
 }
 
 #if 0
@@ -170,6 +170,8 @@ position_menu (GtkMenu *menu, gint *x, gint *y,
     GtkWidget *widget = GTK_WIDGET (user_data);
     GdkScreen *screen;
     gint twidth, theight, tx, ty;
+    GtkAllocation allocation;
+    GtkRequisition requisition;
     GtkTextDirection direction;
     GdkRectangle monitor;
     gint monitor_num;
@@ -182,31 +184,33 @@ position_menu (GtkMenu *menu, gint *x, gint *y,
 
     direction = gtk_widget_get_direction (widget);
 
-    twidth = GTK_WIDGET (menu)->requisition.width;
-    theight = GTK_WIDGET (menu)->requisition.height;
+    gtk_widget_get_requisition (GTK_WIDGET (menu), &requisition);
+    twidth = requisition.width;
+    theight = requisition.height;
 
     screen = gtk_widget_get_screen (GTK_WIDGET (menu));
-    monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+    monitor_num = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (widget));
     if (monitor_num < 0) monitor_num = 0;
     gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
-    if (!gdk_window_get_origin (widget->window, &tx, &ty)) {
+    if (!gdk_window_get_origin (gtk_widget_get_window (widget), &tx, &ty)) {
 	g_warning ("Menu not on screen");
 	return;
     }
 
-    tx += widget->allocation.x;
-    ty += widget->allocation.y;
+    gtk_widget_get_allocation (widget, &allocation);
+    tx += allocation.x;
+    ty += allocation.y;
 
     if (direction == GTK_TEXT_DIR_RTL)
-	tx += widget->allocation.width - twidth;
+	tx += allocation.width - twidth;
 
-    if ((ty + widget->allocation.height + theight) <= monitor.y + monitor.height)
-	ty += widget->allocation.height;
+    if ((ty + allocation.height + theight) <= monitor.y + monitor.height)
+	ty += allocation.height;
     else if ((ty - theight) >= monitor.y)
 	ty -= theight;
-    else if (monitor.y + monitor.height - (ty + widget->allocation.height) > ty)
-	ty += widget->allocation.height;
+    else if (monitor.y + monitor.height - (ty + allocation.height) > ty)
+	ty += allocation.height;
     else
 	ty -= theight;
 
@@ -241,10 +245,10 @@ drive_button_key_press (GtkWidget      *widget,
     DriveButton *self = DRIVE_BUTTON (widget);
 
     switch (event->keyval) {
-    case GDK_KP_Space:
-    case GDK_space:
-    case GDK_KP_Enter:
-    case GDK_Return:
+    case GDK_KEY_KP_Space:
+    case GDK_KEY_space:
+    case GDK_KEY_KP_Enter:
+    case GDK_KEY_Return:
 	drive_button_ensure_popup (self);
 	if (self->popup_menu) {
 	    gtk_menu_popup (GTK_MENU (self->popup_menu), NULL, NULL,
@@ -323,8 +327,8 @@ drive_button_update (gpointer user_data)
 
     /* if no volume or mount, unset image */
     if (!self->volume && !self->mount) {
-	if (GTK_BIN (self)->child != NULL)
-	    gtk_image_set_from_pixbuf (GTK_IMAGE (GTK_BIN (self)->child), NULL);
+	if (gtk_bin_get_child (GTK_BIN (self)) != NULL)
+	    gtk_image_set_from_pixbuf (GTK_IMAGE (gtk_bin_get_child (GTK_BIN (self))), NULL);
 	return FALSE;
     }
 
@@ -358,7 +362,7 @@ drive_button_update (gpointer user_data)
 
     /* base the icon size on the desired button size */
     gtk_widget_size_request (GTK_WIDGET (self), &button_req);
-    gtk_widget_size_request (GTK_BIN (self)->child, &image_req);
+    gtk_widget_size_request (gtk_bin_get_child (GTK_BIN (self)), &image_req);
     width = self->icon_size - (button_req.width - image_req.width);
     height = self->icon_size - (button_req.height - image_req.height);
 
@@ -384,7 +388,7 @@ drive_button_update (gpointer user_data)
 	pixbuf = scaled;
     }
 
-    gtk_image_set_from_pixbuf (GTK_IMAGE (GTK_BIN (self)->child), pixbuf);
+    gtk_image_set_from_pixbuf (GTK_IMAGE (gtk_bin_get_child (GTK_BIN (self))), pixbuf);
     g_object_unref (pixbuf);
 
     gtk_widget_size_request (GTK_WIDGET (self), &button_req);
@@ -454,7 +458,7 @@ static void
 drive_button_reset_popup (DriveButton *self)
 {
     if (self->popup_menu)
-	gtk_object_destroy (GTK_OBJECT (self->popup_menu));
+	gtk_widget_destroy (GTK_WIDGET (self->popup_menu));
     self->popup_menu = NULL;
 }
 
@@ -514,10 +518,11 @@ open_drive (DriveButton *self, GtkWidget *item)
 {
     GdkScreen *screen;
     GtkWidget *dialog;
+    GFile *file;
+    GList *files;
+    GdkAppLaunchContext *launch_context;
+    GDesktopAppInfo *app_info;
     GError *error = NULL;
-    char *argv[3] = { "nautilus", NULL, NULL };
-
-    screen = gtk_widget_get_screen (GTK_WIDGET (self));
 
     if (self->volume) {
 	GMount *mount;
@@ -528,36 +533,53 @@ open_drive (DriveButton *self, GtkWidget *item)
 
 	    file = g_mount_get_root (mount);
 
-	    argv[1] = g_file_get_uri (file);
-	    g_object_unref(file);
-
 	    g_object_unref(mount);
 	}
     } else if (self->mount) {
 	GFile *file;
 
 	file = g_mount_get_root (self->mount);
-	argv[1] = g_file_get_uri (file);
-	g_object_unref(file);
     } else
 	g_return_if_reached();
 
-    if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
-			      G_SPAWN_SEARCH_PATH,
-			      NULL, NULL, NULL, &error)) {
+    app_info = g_desktop_app_info_new ("nautilus");
+
+    if (app_info) {
+	  launch_context = gdk_app_launch_context_new ();
+	  screen = gtk_widget_get_screen (GTK_WIDGET (self));
+	  gdk_app_launch_context_set_screen (launch_context, screen);
+	  files = g_list_prepend (NULL, file);
+	  g_app_info_launch (G_APP_INFO (app_info),
+	                     files,
+	                     G_APP_LAUNCH_CONTEXT (launch_context),
+	                     &error);
+
+	  g_object_unref (launch_context);
+	  g_list_free (files);
+      }
+
+    if (!app_info || error) {
 	dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))),
 						     GTK_DIALOG_DESTROY_WITH_PARENT,
 						     GTK_MESSAGE_ERROR,
 						     GTK_BUTTONS_OK,
-						     _("Cannot execute '%s'"),
-						     argv[0]);
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), error->message, NULL);
+						     _("Cannot start Nautilus File Manager"));
+	  if (error)
+	      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	                                                error->message,
+	                                                NULL);
+	  else
+	      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	                                                _("Could not find Nautilus"),
+	                                                NULL);
+
 	g_signal_connect (dialog, "response",
-			  G_CALLBACK (gtk_object_destroy), NULL);
+			  G_CALLBACK (gtk_widget_destroy), NULL);
 	gtk_widget_show (dialog);
 	g_error_free (error);
     }
-    g_free (argv[1]);
+
+    g_object_unref(file);
 }
 
 /* copied from gnome-volume-manager/src/manager.c maybe there is a better way than
@@ -768,13 +790,13 @@ unmount_drive (DriveButton *self, GtkWidget *item)
 	mount = g_volume_get_mount (self->volume);
 	if (mount)
 	{
-	    g_mount_unmount (mount, G_MOUNT_UNMOUNT_NONE,
-			     NULL, NULL, NULL);
+	    g_mount_unmount_with_operation (mount, G_MOUNT_UNMOUNT_NONE,
+	                                    NULL, NULL, NULL, NULL);
 	    g_object_unref (mount);
 	}
     } else if (self->mount) {
-	g_mount_unmount (self->mount, G_MOUNT_UNMOUNT_NONE,
-			 NULL, NULL, NULL);
+	g_mount_unmount_with_operation (self->mount, G_MOUNT_UNMOUNT_NONE,
+			                NULL, NULL, NULL, NULL);
     } else {
 	g_return_if_reached();
     }
@@ -791,11 +813,15 @@ static void
 eject_drive (DriveButton *self, GtkWidget *item)
 {
     if (self->volume) {
-	g_volume_eject (self->volume, G_MOUNT_UNMOUNT_NONE,
-			NULL,(GAsyncReadyCallback) eject_finish, NULL);
+	g_volume_eject_with_operation (self->volume, G_MOUNT_UNMOUNT_NONE,
+	                               NULL, NULL,
+	                               (GAsyncReadyCallback) eject_finish,
+	                               NULL);
     } else if (self->mount) {
-	g_mount_eject (self->mount, G_MOUNT_UNMOUNT_NONE,
-		       NULL, (GAsyncReadyCallback) eject_finish, NULL);
+	g_mount_eject_with_operation (self->mount, G_MOUNT_UNMOUNT_NONE,
+	                              NULL, NULL,
+	                              (GAsyncReadyCallback) eject_finish,
+	                              NULL);
     } else {
 	g_return_if_reached();
     }
