@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 
-import gtk, gtk.gdk
-import gobject
+from gi.repository import GObject, Gtk, Gdk, GdkPixbuf, Gio
 import os
 import invest
 from gettext import gettext as _
@@ -15,16 +14,16 @@ import time
 AUTOREFRESH_TIMEOUT = 20*60*1000 # 15 minutes
 
 # based on http://www.johnstowers.co.nz/blog/index.php/2007/03/12/threading-and-pygtk/
-class _IdleObject(gobject.GObject):
+class _IdleObject(GObject.Object):
 	"""
-	Override gobject.GObject to always emit signals in the main thread
+	Override GObject.Object to always emit signals in the main thread
 	by emmitting on an idle handler
 	"""
 	def __init__(self):
-		gobject.GObject.__init__(self)
+		GObject.Object.__init__(self)
 
 	def emit(self, *args):
-		gobject.idle_add(gobject.GObject.emit,self,*args)
+		GObject.idle_add(GObject.Object.emit,self,*args)
 
 class ImageRetriever(Thread, _IdleObject):
 	"""
@@ -33,7 +32,7 @@ class ImageRetriever(Thread, _IdleObject):
 	"""
 	__gsignals__ =  { 
 			"completed": (
-				gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
+				GObject.SignalFlags.RUN_LAST, GObject.TYPE_NONE, []),
 			# FIXME: should we be making use of this?
 			#"progress": (
 			#	gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
@@ -47,13 +46,12 @@ class ImageRetriever(Thread, _IdleObject):
 		self.retrieved = False
 		
 	def run(self):
-		self.image = gtk.Image()
+		self.image = Gtk.Image()
 		try: sock = urllib.urlopen(self.image_url, proxies = invest.PROXY)
-		except:
-			if invest.DEBUGGING:
-				print "Error while opening %s" % self.image_url
+		except Exception, msg:
+			invest.debug("Error while opening %s: %s" % (self.image_url, msg))
 		else:
-			loader = gtk.gdk.PixbufLoader()
+			loader = GdkPixbuf.PixbufLoader()
 			loader.connect("closed", lambda loader: self.image.set_from_pixbuf(loader.get_pixbuf()))
 			loader.write(sock.read())
 			sock.close()
@@ -100,14 +98,18 @@ class FinancialChart:
 	def __init__(self, ui):
 		self.ui = ui
 		
+		#Time ranges of the plot
+		self.time_ranges = ["1d", "5d", "3m", "6m", "1y", "5y", "my"]
+		
 		# Window Properties
 		win = ui.get_object("window")
 		win.set_title(_("Financial Chart"))
 		
 		try:
-			pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(join(invest.ART_DATA_DIR, "invest_neutral.svg"), 96,96)
+			pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(join(invest.ART_DATA_DIR, "invest_neutral.svg"), 96,96)
 			self.ui.get_object("plot").set_from_pixbuf(pixbuf)
 		except Exception, msg:
+			invest.debug("Could not load 'invest-neutral.svg' file: %s" % msg)
 			pass
 		
 		# Defaut comboboxes values
@@ -203,10 +205,10 @@ class FinancialChart:
 				a += "%s%s," % (name[1:], param)
 		
 		# Create the image URL -----------------------------------------------------
-		chart_base_url = "http://ichart.europe.yahoo.com/z?s=%(s)s&t=%(t)s&q=%(q)s&l=%(l)s&z=%(z)s&p=%(p)s&a=%(a)s%(opt)s"
+		chart_base_url = "http://chart.finance.yahoo.com/z?s=%(s)s&t=%(t)s&q=%(q)s&l=%(l)s&z=%(z)s&p=%(p)s&a=%(a)s%(opt)s"
 		url = chart_base_url % {
 			"s": tickers[0],
-			"t": self.ui.get_object("t").get_active_text(),
+			"t": self.time_ranges[self.ui.get_object("t").get_active()],
 			"q": self.ui.get_object("q").get_active_text(),
 			"l": "off",
 			"z": "l",
@@ -238,14 +240,14 @@ class FinancialChart:
 	
 	def on_autorefresh_toggled(self, autorefresh):
 		if self.autorefresh_id != 0:
-			gobject.source_remove(self.autorefresh_id)
+			GObject.source_remove(self.autorefresh_id)
 			self.autorefresh_id = 0
 			
 		if autorefresh.get_active():
-			self.autorefresh_id = gobject.timeout_add(AUTOREFRESH_TIMEOUT, self.on_refresh_chart, True)
+			self.autorefresh_id = GObject.timeout_add(AUTOREFRESH_TIMEOUT, self.on_refresh_chart, True)
 
 def show_chart(tickers):
-	ui = gtk.Builder();
+	ui = Gtk.Builder();
 	ui.add_from_file(os.path.join(invest.BUILDER_DATA_DIR, "financialchart.ui"))
 	chart = FinancialChart(ui)
 	ui.get_object("s").set_text(' '.join(tickers))
